@@ -1,13 +1,8 @@
 
 
 """
-Crocodile Philosophy:
-Make Python even friendlier, by making available the common functionality for everyday use, e.g., path management, file management
-At the risk of vandalizing the concept, Crocodile is about making Python more MATLAB-like, in that more libraries are loaded up at
-start time than mere basic arithmetic, but just enought to make it more useful for everyday errands. Thus, the terseness of Crocodile makes Python REPL a proper shell
-In implementation, the focus is on ease of use, not efficiency.
 """
-
+import argparse
 from crocodile.core import *
 from crocodile.file_management import *
 from crocodile.meta import *
@@ -24,3 +19,16 @@ print(f"Crocodile Shell {__import__('crocodile').__version__}")
 # link to tutorial or github
 print(f"Made with ❤️")
 print(P(__file__).parent.joinpath("art").search().sample(size=1)[0].read_text())
+
+
+def build_parser():
+    parser = argparse.ArgumentParser(description="Generic Parser to launch a script in a separate window.")
+    parser.add_argument("--cmd", "-c", dest="cmd", help="Python command.", default="")
+    args = parser.parse_args()
+    # tb.Struct(args.__dict__).print(as_config=True)
+    print(args.cmd)
+    exec(args.cmd, globals())
+
+
+if __name__ == "__main__":
+    build_parser()
