@@ -175,8 +175,8 @@ class DataReader(tb.Base):
         if strings is None: strings = ["x", "y"]
         self.split.update({astring + '_train': result[ii * 2] for ii, astring in enumerate(strings)})
         self.split.update({astring + '_test': result[ii * 2 + 1] for ii, astring in enumerate(strings)})
-        self.specs.ip_shape = self.split.x_train.shape[1:]  # useful info for instantiating models.
-        self.specs.op_shape = self.split.y_train.shape[1:]  # useful info for instantiating models.
+        self.specs.ip_shape = self.split.x_train[0].shape  # useful info for instantiating models.
+        self.specs.op_shape = self.split.y_train[0].shape  # useful info for instantiating models.
         print(f"================== Training Data Split ===========================")
         self.split.print()
 
@@ -359,7 +359,7 @@ class BaseModel(ABC):
             if hasattr(a_metric, "path"): name = a_metric.name
             elif hasattr(a_metric, "__name__"): name = a_metric.__name__
             else: name = "unknown"
-            # try:  # EAFP principle.
+            # try:  # EAFP vs LBYL: both are duck-typing styles as they ask for what object can do (whether by introspection or trial) as opposed to checking its type.
             #     path = a_metric.path  # works for subclasses Metrics
             # except AttributeError: path = a_metric.__name__  # works for functions.
             loss_dict[name] = []
