@@ -1,6 +1,7 @@
 
 import crocodile.toolbox as tb
 import platform
+import getpass
 import os
 import sys
 
@@ -9,14 +10,13 @@ L = tb.L
 
 system = platform.system()
 OS = os.getenv("OS")  # Windows_NT
-sep = ";" if system == "Windows" else ":"  # path separator, not to be confused with P.sep
+sep = ";" if system == "Windows" else ":"  # PATH separator, this is special for PATH object, not to be confused with P.sep (normal paths).
 env = tb.Struct(dict(os.environ)).clean_view
 exe = P(sys.executable)
 
 tm = tb.Terminal()
 
 # ============================== Common Paths ============================
-DotFiles = P.home().joinpath("dotfiles")
 
 LocalAppData = P(tmp) if (tmp := os.getenv("LOCALAPPDATA")) else None  # C:\Users\username\AppData\Local
 AppData = P(tmp) if (tmp := os.getenv("APPDATA")) else None  # C:\Users\username\AppData\Roaming
@@ -37,20 +37,25 @@ Temp = Tmp
 Path = L(os.getenv("PATH").split(sep)).apply(P)
 PSPath = L(tmp.split(sep)).apply(P) if (tmp := os.getenv("PSModulePath")) else None
 
-HostName = os.getenv("COMPUTERNAME")  # e.g. "MY-SURFACE"
-LogonServer = os.getenv("LOGONSERVER")  # e.g. "\\MY-SURFACE"
-UserDomain = os.getenv("USERDOMAIN")  # e.g. HAD OR MY-SURFACE
+HostName          = platform.node()  # e.g. "MY-SURFACE", os.getenv("COMPUTERNAME") only works for windows.
+UserName          = getpass.getuser()  # e.g: username, os.getenv("USERNAME") only works for windows.
+UserDomain        = os.getenv("USERDOMAIN")  # e.g. HAD OR MY-SURFACE
 UserDomainRoaming = P(tmp) if (tmp := os.getenv("USERDOMAIN_ROAMINGPROFILE")) else None  # e.g. SURFACE
-UserName = os.getenv("USERNAME")  # e.g: username
-UserProfile = P(tmp) if (tmp := os.getenv("USERPROFILE")) else None  # e.g C:\Users\username
-HomePath = P(tmp) if (tmp := os.getenv("HOMEPATH")) else None  # e.g. C:\Users\username
-Public = P(tmp) if (tmp := os.getenv("PUBLIC")) else None  # C:\Users\Public
+LogonServer       = os.getenv("LOGONSERVER")  # e.g. "\\MY-SURFACE"
+# UserProfile       = P(tmp) if (tmp := os.getenv("USERPROFILE")) else None  # e.g C:\Users\username
+# HomePath          = P(tmp) if (tmp := os.getenv("HOMEPATH")) else None  # e.g. C:\Users\username
+Public            = P(tmp) if (tmp := os.getenv("PUBLIC")) else None  # C:\Users\Public
+
+WSL_HOME = P(rf"\\wsl$\Ubuntu\home")
+WIN_HOME = P(rf"/mnt/c/Users")
+
 
 OneDriveConsumer = P(tmp) if (tmp := os.getenv("OneDriveConsumer")) else None
 OneDriveCommercial = P(tmp) if (tmp := os.getenv("OneDriveCommercial")) else None
 OneDrive = P(tmp) if (tmp := os.getenv("OneDrive")) else None
 OneDriveExe = LocalAppData.joinpath("Microsoft/OneDrive/OneDrive.exe") if LocalAppData else None
 
+DotFiles = P.home().joinpath("dotfiles")
 
 # ============================== Networking ==============================
 
@@ -66,9 +71,7 @@ def get_address():
     # else: return mac
     import socket
     local_ip_v4 = socket.gethostbyname(socket.gethostname())
-    from requests import get
-    public_ip = get('https://api.ipify.org').text
-    return dict(subnet_mask=subnet_mask, mac_address=mac_address, local_ip_v4=local_ip_v4, default_gateway=default_gateway, public_ip=public_ip)
+    return dict(subnet_mask=subnet_mask, mac_address=mac_address, local_ip_v4=local_ip_v4, default_gateway=default_gateway, public_ip=P('https://api.ipify.org').download(memory=True).text)
 
 
 # ============================== System Variables ==============================
@@ -98,12 +101,9 @@ class EnvVar:
             if temp is False:
                 res = f"setx {key} {val}"  # WARNING: setx limits val to 1024 characters # in case the variable included ";" separated paths, this limit can be exceeded.
                 return res if not run else tm.run(res, shell="powershell")
-            else:
-                raise NotImplementedError
-        elif system == "Linux":
-            return f"export {key} = {val}"  # this is shell command. in csh: `setenv key val`
-        else:
-            raise NotImplementedError
+            else: raise NotImplementedError
+        elif system == "Linux": return f"export {key} = {val}"  # this is shell command. in csh: `setenv key val`
+        else: raise NotImplementedError
 
     @staticmethod
     def get(key, run=False):
@@ -111,7 +111,6 @@ class EnvVar:
         return result if run is False else tm.run(result, shell="powershell")
 
     # in windows cmd `%key%`
-
     @staticmethod
     def delete(key, temp=True, scope=["User", "system"][0], run=False):
         if system == "Windows":
@@ -130,6 +129,9 @@ class PathVar:
     def append_temporarily(path, kind="append", run=False):
         if system == "Windows":
             """Source: https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/about/about_environment_variables?view=powershell-7.2"""
+            if tb.P(path) in Path:
+                print(f"Path passed `{path}` is already in PATH, skipping the appending.")
+                return None
             if kind == "append": command = fr'$env:Path += ";{path}"'  # Append to the Path variable in the current window:
             elif kind == "prefix": command = fr'$env:Path = "{path};" + $env:Path'  # Prefix the Path variable in the current window:
             elif kind == "replace": command = fr'$env:Path = "{path}"'  # Replace the Path variable in the current window (use with caution!):
@@ -142,22 +144,23 @@ class PathVar:
     def append_permanently(path, scope=["User", "system"][0], run=False):
         if system == "Windows":
             # AVOID THIS AND OPT TO SAVE IT IN $profile.
-            backup = fr'$env:PATH >> {tb.P.tmpfile()}.path_backup; '
+            tmp_path = tb.P.tmpfile(suffix=".path_backup")
+            if tb.P(path) in Path:
+                print(f"Path passed `{path}` is already in PATH, skipping the appending.")
+                return None
+            backup = fr'$env:PATH >> {tmp_path}; '
             command = fr'[Environment]::SetEnvironmentVariable("Path", $env:PATH + ";{path}", "{scope}")'
             result = backup + command
-            return result if run is False else tm.run(result, shell="powershell")
+            return result if run is False else tm.run(result, shell="powershell").print()
         else: tb.P.home().joinpath(".bashrc").append_text(f"export PATH='{path}:$PATH'")
 
     @staticmethod
     def set_permanetly(path, scope=["User", "system"][0], run=False):
-        """This is useful if path is manipulated with a text editor or Python string manipulation
-        (not recommended programmatically even if original is backed up) and set the final value.
-        On a windows machine, system and user variables are kept separately. env:Path returns the combination
-        of both, starting from system then user.
-        To see impact of change, you will need to restart the process from which the shell started. This is probably
-        windows explorer. This can be achieved by suspending the process, alternatively you need to logoff and on.
-        This is because enviroment variables are inherited from parent process, and so long explorere is not updated,
-        restarting the shell would not help."""
+        """This is useful if path is manipulated with a text editor or Python string manipulation (not recommended programmatically even if original is backed up) and set the final value.
+        On a windows machine, system and user variables are kept separately. env:Path returns the combination of both, starting from system then user.
+        To see impact of change, you will need to restart the process from which the shell started. This is probably windows explorer.
+        This can be achieved by suspending the process, alternatively you need to logoff and on.
+        This is because enviroment variables are inherited from parent process, and so long explorere is not updated, restarting the shell would not help."""
         tmpfile = tb.P.tmpfile(suffix=".path_backup")
         print(f"Saving original path to {tmpfile}")
         backup = fr'$env:PATH >> {tmpfile}; '
@@ -187,7 +190,7 @@ def get_shell_profiles(shell):
 
 
 def construct_path(path_list): return tb.L(__import__("pd").unique(path_list)).reduce(lambda x, y: str(x) + sep + str(y))
-def get_defined_programs(string_="*.exe"): return tb.P.env().Path.search(string_).reduce(lambda x, y: x+y).print()
+def get_path_defined_files(string_="*.exe"): res = tb.P.get_env().Path.search(string_).reduce(lambda x, y: x + y); res.print(); return res
 
 
 if __name__ == '__main__':
