@@ -13,13 +13,14 @@ from crocodile import file_management as _fm
 # from crocodile import run
 
 # CORE =====================================
-str2timedelta, timestamp, randstr, validate_name, install_n_import = core.str2timedelta, core.timestamp, core.randstr, core.validate_name, core.install_n_import
+str2timedelta, timestamp, randstr, validate_name, install_n_import, get_env= core.str2timedelta, core.timestamp, core.randstr, core.validate_name, core.install_n_import, core.get_env
 Base, Struct, Display, Save, List = core.Base, core.Struct, core.Display, core.Save, core.List
 # File Management ==========================
-P, Read, Compression, Fridge, encrypt, decrypt = _fm.P, _fm.Read, _fm.Compression, _fm.Fridge, _fm.encrypt, _fm.decrypt
+P, Read, Compression, Cache, encrypt, decrypt = _fm.P, _fm.Read, _fm.Compression, _fm.Cache, _fm.encrypt, _fm.decrypt
 # META ====================================
 Experimental, Terminal, Log, Null, Scheduler, SSH = meta.Experimental, meta.Terminal, meta.Log, meta.Null, meta.Scheduler, meta.SSH
 
+Path = P
 L = List
 S = Struct
 D = Display
@@ -29,9 +30,8 @@ tmp = P.tmp
 
 _ = Base, timestamp, Save, Terminal, List, Struct, Display, P, Read, Compression, Experimental
 # from crocodile.core import datetime, dt, os, sys, string, random, np, copy, dill
-logging, subprocess, sys = meta.logging, meta.subprocess, meta.sys
+logging, subprocess, sys, time = meta.logging, meta.subprocess, meta.sys, meta.time
 datetime = _fm.datetime
-
 # _ = False
 # if _:
 #     from crocodile import matplotlib_management as _pm
